package javax.naming.directory;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.naming.OperationNotSupportedException;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import java.lang.reflect.Array;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import java.util.Vector;

public class BasicAttribute implements Attribute
{
    protected String attrID;
    protected transient Vector<Object> values;
    protected boolean ordered;
    private static final long serialVersionUID = 6743528196119291326L;
    
    @Override
    public Object clone() {
        BasicAttribute basicAttribute;
        try {
            basicAttribute = (BasicAttribute)super.clone();
        }
        catch (CloneNotSupportedException ex) {
            basicAttribute = new BasicAttribute(this.attrID, this.ordered);
        }
        basicAttribute.values = (Vector)this.values.clone();
        return basicAttribute;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o != null && o instanceof Attribute) {
            Attribute attribute = (Attribute)o;
            if (this.isOrdered() != attribute.isOrdered()) {
                return false;
            }
            int size;
            if (this.attrID.equals(attribute.getID()) && (size = this.size()) == attribute.size()) {
                try {
                    if (this.isOrdered()) {
                        int i = 0;
                        while (i < size) {
                            if (!valueEquals(this.get(i), attribute.get(i))) {
                                return false;
                            }
                            ++i;
                        }
                    }
                    else {
                        NamingEnumeration<Object> all = attribute.getAll();
                        while (all.hasMoreElements()) {
                            if (this.find(all.nextElement()) >= 0) {
                                continue;
                            }
                            return false;
                        }
                    }
                }
                catch (NamingException ex) {
                    return false;
                }
                return true;
            }
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        int hashCode = this.attrID.hashCode();
        for (int size = this.values.size(), i = 0; i < size; ++i) {
            Object element = this.values.elementAt(i);
            if (element != null) {
                if (element.getClass().isArray()) {
                    for (int length = Array.getLength(element), j = 0; j < length; ++j) {
                        Object value = Array.get(element, j);
                        if (value != null) {
                            hashCode += value.hashCode();
                        }
                    }
                }
                else {
                    hashCode += element.hashCode();
                }
            }
        }
        return hashCode;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer(this.attrID + ": ");
        if (this.values.size() == 0) {
            sb.append("No values");
        }
        else {
            int n = 1;
            for (Enumeration elements = this.values.elements(); elements.hasMoreElements(); n = 0) {
                if (n == 0) {
                    sb.append(", ");
                }
                sb.append(elements.nextElement());
            }
        }
        return sb.toString();
    }
    
    public BasicAttribute(String s) {
        this(s, false);
    }
    
    public BasicAttribute(String s, Object o) {
        this(s, o, false);
    }
    
    public BasicAttribute(String attrID, boolean ordered) {
        this.ordered = false;
        this.attrID = attrID;
        this.values = new Vector();
        this.ordered = ordered;
    }
    
    public BasicAttribute(String s, Object obj, boolean b) {
        this(s, b);
        this.values.addElement(obj);
    }
    
    @Override
    public NamingEnumeration<?> getAll() throws NamingException {
        return new BasicAttribute.ValuesEnumImpl();
    }
    
    @Override
    public Object get() throws NamingException {
        if (this.values.size() == 0) {
            throw new NoSuchElementException("Attribute " + this.getID() + " has no value");
        }
        return this.values.elementAt(0);
    }
    
    @Override
    public int size() {
        return this.values.size();
    }
    
    @Override
    public String getID() {
        return this.attrID;
    }
    
    @Override
    public boolean contains(Object o) {
        return this.find(o) >= 0;
    }
    
    private int find(Object o) {
        if (o == null) {
            int size2 = this.values.size();
            int j = 0;
            while (j < size2) {
                if (this.values.elementAt(j) == null) {
                    return j;
                }
                ++j;
            }
        }
        else {
            Class<Object> class1;
            if ((class1 = o.getClass()).isArray()) {
                int size = this.values.size();
                int i = 0;
                while (i < size) {
                    Object element = this.values.elementAt(i);
                    if (element != null && class1 == element.getClass() && arrayEquals(o, element)) {
                        return i;
                    }
                    ++i;
                }
            }
            else {
                return this.values.indexOf(o, 0);
            }
        }
        return -1;
    }
    
    private static boolean valueEquals(Object o, Object obj) {
        if (o == obj) {
            return true;
        }
        if (o == null) {
            return false;
        }
        if (o.getClass().isArray() && obj.getClass().isArray()) {
            return arrayEquals(o, obj);
        }
        return o.equals(obj);
    }
    
    private static boolean arrayEquals(Object o, Object o2) {
        int length;
        if ((length = Array.getLength(o)) != Array.getLength(o2)) {
            return false;
        }
        for (int i = 0; i < length; ++i) {
            Object value = Array.get(o, i);
            Object value2 = Array.get(o2, i);
            if (value == null || value2 == null) {
                if (value != value2) {
                    return false;
                }
            }
            else if (!value.equals(value2)) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public boolean add(Object obj) {
        if (this.isOrdered() || this.find(obj) < 0) {
            this.values.addElement(obj);
            return true;
        }
        return false;
    }
    
    @Override
    public boolean remove(Object o) {
        int find = this.find(o);
        if (find >= 0) {
            this.values.removeElementAt(find);
            return true;
        }
        return false;
    }
    
    @Override
    public void clear() {
        this.values.setSize(0);
    }
    
    @Override
    public boolean isOrdered() {
        return this.ordered;
    }
    
    @Override
    public Object get(int index) throws NamingException {
        return this.values.elementAt(index);
    }
    
    @Override
    public Object remove(int n) {
        Object element = this.values.elementAt(n);
        this.values.removeElementAt(n);
        return element;
    }
    
    @Override
    public void add(int index, Object obj) {
        if (!this.isOrdered() && this.contains(obj)) {
            throw new IllegalStateException("Cannot add duplicate to unordered attribute");
        }
        this.values.insertElementAt(obj, index);
        return;
    }
    
    @Override
    public Object set(int n, Object obj) {
        if (!this.isOrdered() && this.contains(obj)) {
            throw new IllegalStateException("Cannot add duplicate to unordered attribute");
        }
        Object element = this.values.elementAt(n);
        this.values.setElementAt(obj, n);
        return element;
    }
    
    @Override
    public DirContext getAttributeSyntaxDefinition() throws NamingException {
        throw new OperationNotSupportedException("attribute syntax");
    }
    
    @Override
    public DirContext getAttributeDefinition() throws NamingException {
        throw new OperationNotSupportedException("attribute definition");
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.values.size());
        for (int i = 0; i < this.values.size(); ++i) {
            objectOutputStream.writeObject(this.values.elementAt(i));
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        this.values = new Vector(Math.min(1024, int1));
        while (--int1 >= 0) {
            this.values.addElement(objectInputStream.readObject());
        }
    }
    
    class ValuesEnumImpl implements NamingEnumeration<Object>
    {
        Enumeration<Object> list;
        
        ValuesEnumImpl() {
            this.list = BasicAttribute.this.values.elements();
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.list.hasMoreElements();
        }
        
        @Override
        public Object nextElement() {
            return this.list.nextElement();
        }
        
        @Override
        public Object next() throws NamingException {
            return this.list.nextElement();
        }
        
        @Override
        public boolean hasMore() throws NamingException {
            return this.list.hasMoreElements();
        }
        
        @Override
        public void close() throws NamingException {
            this.list = null;
        }
    }
}
